package com.loafle.overflow.redis.service;

import com.loafle.overflow.commons.model.PublishMessage;
import com.loafle.overflow.commons.service.MessagePublisher;
import com.loafle.overflow.module.domain.model.Domain;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.member.service.MemberService;
import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.ChannelTopic;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class RedisMessagePublisher implements MessagePublisher {

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private MemberService memberService;

    private Map<String, ChannelTopic> topics;

    public RedisMessagePublisher() {
    }

    public RedisMessagePublisher(final RedisTemplate<String, Object> redisTemplate, final Map<String, ChannelTopic> topics) {
        this.redisTemplate = redisTemplate;
        this.topics = topics;
    }

    public void publishToMember(final String channel, final String memberID, final String method, final Object... params) {
        PublishMessage message = new PublishMessage();
        message.addTarget(memberID);
        this.publish(channel, message, method, params);
    }
    public void publishToDomain(final String channel, final Domain domain, final String method, final Object... params) {
        PublishMessage message = new PublishMessage();
        message.setTargets(getMemberListByDomain(domain));
        this.publish(channel, message, method, params);

    }
    public void publishToDomainByProbeKey(final String channel, final String probeKey, final String method, final Object... params) {
        PublishMessage message = new PublishMessage();
        message.setTargets(getMemberListByProbeKey(probeKey));
        this.publish(channel, message, method, params);
    }

    public void publishToProbe(final String channel, final String key, final String method, final Object... params) {
        PublishMessage message = new PublishMessage();
        message.addTarget(key);
        this.publish(channel, message, method, params);
    }

    @Cacheable("memberListByDomain")
    protected List<String> getMemberListByDomain(final Domain domain) {
        return this.getMemberList(memberService.readAllByDomain(domain));
    }

    @Cacheable("memberListByProbeKey")
    protected List<String> getMemberListByProbeKey(final String probeKey) {
        return this.getMemberList(memberService.readAllByProbeKey(probeKey));
    }

    protected List<String> getMemberList(final List<Member> members) {
        List<String> results = new ArrayList<>(members.size());
        for (Member member: members) {
            results.add(member.getEmail());
        }

        return results;
    }


    protected void publish(final String channel, PublishMessage message, final String method, Object... params) {
        ChannelTopic topic = this.topics.get(channel);
        message.setMessage(new PublishMessage.PublishMessageBody(method, this.getMessageBody(params)));

        try {
            String json = this.objectMapper.writeValueAsString(message);
            redisTemplate.convertAndSend(topic.getTopic(), json);
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    protected List<String> getMessageBody(final Object... params) {
        List<String> results = new ArrayList<>(params.length);
        try {
            for (Object param : params) {
                if (param.getClass().equals(String.class)) {
                    results.add((String)param);
                } else {
                    results.add(objectMapper.writeValueAsString(param));
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return results;
    }

//    public void publishToMember(final String message) {
//        redisTemplate.convertAndSend(topic.getTopic(), message);
//    }
//    public void publishToDomain(final String message) {
//        redisTemplate.convertAndSend(topic.getTopic(), message);
//    }
}
